//------------------------------------------------------------------------------
//
//  npgl.c
//  ANTZ application based on NPE
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------


#include "npgl.h"

#include "../SOIL.h"

#include "../npctrl.h"

#include "../os/npos.h"

#include "watchtools.h"
#include "windows.h"

GLuint texture;														// texture, debug zz

int previousX;
int previousY;
int picking;
//MB zz
float theta;
float zeta;
float cmDX;
float cmDY;

#define NOBUTTON -1
int buttonDown;
//MB zz end

void npGLResizeScene (int width, int height);
void npCopyGPUtoCPU (void* dataRef);
void npCameraUpdate( void* dataRef );							//MB zz


void npPick( int x, int y );											//debug, zz //MB//

//------------------------------------------------------------------------------
void npInitGL(void* dataRef) 
{
	previousX = 0;
	previousY = 0;

	picking = false;

	theta = 0.0;
	zeta  = 0.0;
	cmDX = 0.0;
	cmDY = 0.0;

	buttonDown = NOBUTTON;


	npInitGLDraw (dataRef);

	npInitGLDraw (dataRef);

//	glShadeModel (GL_SMOOTH);
//	glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
//	glClearDepth (1.0f);

	glEnable (GL_DEPTH_TEST);

//	glDepthFunc (GL_LEQUAL);
//	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	  // load our texture												// texture, debug zz
//  texture = LoadTextureRAW( "texture.raw", TRUE );
  texture = SOIL_load_OGL_texture
	(
		"map00001.jpg",
		SOIL_LOAD_AUTO,
		SOIL_CREATE_NEW_ID,
		SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT
	);																// texture, debug zz

}


// currently just single window support, add multi-screen capability later... debug zz
// redbook has info on how to calculate the projections
//------------------------------------------------------------------------------
void npGLResizeScene (int wd, int ht)							
{
	int camIndex = 1;	//update with list in data->io->view.camIndex[]		debug zz
	GLfloat ratio = 1.0;

	pData data = npDataGetDataRef();

	NPplotPtr camPlot = data->map.plot[camIndex];
	NPcameraPtr camData = camPlot->data;

	//MB//						//was width, made global by MB, undo it... debug, zz
	width  = wd;
	height = ht;
	//MB-end//

	if (height > 0)							// Prevent a divide by zero
		camData->aspectRatio = (GLfloat)width / (GLfloat)height;

	// Reset the coordinate system before modifying
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	
	// Set the viewport to be the entire window
    glViewport(0, 0, width, height);

	// Set the clipping volume
	gluPerspective (camData->fov, camData->aspectRatio, 
					camData->clipNear, camData->clipFar);
	glMatrixMode (GL_MODELVIEW);

	npGLDrawScene (data);
}


//------------------------------------------------------------------------------
void npGLDrawScene (void* dataRef)
{
	int	camIndex = 1;
	GLfloat upX = 0.0f, upY = 0.0f, upZ = 1.0f;
	
	GLfloat ambient[] = { 0.3f, 0.3f, 0.3f, 1.0f };
	GLfloat diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat specular[] = { 0.1f, 0.1f, 0.1f, 0.5f };
	GLfloat position[] = { -150.0f, 150.0f, -150.0f, 1.0f };
	GLfloat positionB[] = { 150.0f, -150.0f, 150.0f, 1.0f };
	GLfloat positionC[] = { 0.0f, 0.0f, 15.0f, 1.0f };

	GLfloat globalAmbient[] = { 0.3f, 0.3f, 0.3f, 1.0f };

	GLfloat materialSpecular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat materialShininess[] = { 128.0f };
	GLfloat materialAmbient[] = { 0.25f, 0.25f, 0.25f, 1.0f };
	GLfloat materialDiffuse[] = { 0.4f, 0.4f, 0.4f, 1.0f };
	GLfloat localAmbient[] = { 0.0f, 0.0f, 0.0f, 1.0f };


	GLfloat angle = 0.0;
	GLfloat redDiffuseMaterial[] = {1.0f, 1.0f, 1.0f}; //set the material to red
	GLfloat whiteSpecularMaterial[] = {1.0f, 1.0f, 1.0f}; //set the material to white
	GLfloat greenEmissiveMaterial[] = {0.1f, 0.1f, 0.1f}; //set the material to green
	GLfloat whiteSpecularLight[] = {1.0f, 1.0f, 1.0f}; //set the light specular to white
	GLfloat blackAmbientLight[] = {0.0f, 0.0f, 0.0f}; //set the light ambient to black
	GLfloat whiteDiffuseLight[] = {1.0f, 1.0f, 1.0f}; //set the diffuse light to white
	GLfloat blankMaterial[] = {0.0f, 0.0f, 0.0f}; //set the diffuse light to white
	GLfloat mShininess[] = {128}; //set the shininess of the material

	GLfloat redNotGreenNotBlue[] = {1.0f, -1.0f, -1.0f}; //set the material to red

	pData data = (pData) dataRef;
	NPplotPtr camPlot = data->map.plot[camIndex];
	NPcameraPtr camData = camPlot->data;

	NPplotPtr gridPlot = data->map.plot[2];					// texture, debug zz
	NPgridPtr gridData = gridPlot->data;

	GLfloat gridX = 0.0f, gridY = 0.0f;

	//lighting
//	glLightModelfv (GL_LIGHT_MODEL_AMBIENT, globalAmbient);
	
	npCtrlUpdate (dataRef);	  //update data, positions, physics, interactions...

	glClearColor (data->io.clear.r, data->io.clear.g, data->io.clear.b,
					data->io.clear.a);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

//	glfwSwapInterval(3);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);

	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);
	glEnable (GL_LIGHT1);
	glEnable (GL_LIGHT2);

	glLightfv (GL_LIGHT0, GL_AMBIENT, ambient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv (GL_LIGHT0, GL_SPECULAR, specular);
	glLightfv (GL_LIGHT0, GL_POSITION, position);
 
	glLightfv(GL_LIGHT0, GL_SPECULAR, whiteSpecularLight);
    glLightfv(GL_LIGHT0, GL_AMBIENT, blackAmbientLight);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, whiteDiffuseLight);

	glLightfv (GL_LIGHT1, GL_AMBIENT, ambient);
	glLightfv (GL_LIGHT1, GL_DIFFUSE, diffuse);
	glLightfv (GL_LIGHT1, GL_SPECULAR, specular);
	glLightfv (GL_LIGHT1, GL_POSITION, positionB);
 
	glLightfv(GL_LIGHT1, GL_SPECULAR, whiteSpecularLight);
    glLightfv(GL_LIGHT1, GL_AMBIENT, blackAmbientLight);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, whiteDiffuseLight);

	glLightfv (GL_LIGHT2, GL_AMBIENT, blackAmbientLight);
	glLightfv (GL_LIGHT2, GL_DIFFUSE, redNotGreenNotBlue);
	glLightfv (GL_LIGHT2, GL_SPECULAR, redNotGreenNotBlue);
	glLightfv (GL_LIGHT2, GL_POSITION, positionC);
 
	glLightfv(GL_LIGHT2, GL_SPECULAR, redNotGreenNotBlue);
    glLightfv(GL_LIGHT2, GL_AMBIENT, blackAmbientLight);
    glLightfv(GL_LIGHT2, GL_DIFFUSE, redNotGreenNotBlue);
/*
	glMaterialfv ( GL_FRONT, GL_SPECULAR, materialSpecular);
	glMaterialfv ( GL_FRONT, GL_SHININESS, materialShininess);
	glMaterialfv ( GL_FRONT, GL_DIFFUSE, materialDiffuse);
	glMaterialfv ( GL_FRONT, GL_AMBIENT, materialAmbient);
*/
//	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, whiteSpecularMaterial);
//	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mShininess);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, redDiffuseMaterial);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, greenEmissiveMaterial);

	glEnable (GL_COLOR_MATERIAL);

	//glBlendFunc(GL_SRC_COLOR, GL_ONE_MINUS_SRC_ALPHA);
	glBlendFunc(GL_SRC_ALPHA, GL_DST_COLOR);
//	glEnable(GL_BLEND);													// debug, zz
	//glAlphaFunc(GL_GREATER,0.1f);
//	glEnable(GL_ALPHA_TEST);											//zz

	//	glDisable (GL_DEPTH_TEST);


	glShadeModel (GL_SMOOTH);

	npDrawPlots (dataRef);				//draws mesh, graphs, pins, etc...

      // setup texture mapping									// texture, debug zz

	//gridPlot = data->map.plot[2];

	gridX = gridPlot->scale.x * 5.0f * (GLfloat) gridData->segments.x;
	gridY = gridPlot->scale.y * 5.0f * (GLfloat) gridData->segments.y;

	if (!npGetPickPass())	//allow for click on background tex's for mouse flying
	{
	 glDisable (GL_LIGHTING);
      glEnable( GL_TEXTURE_2D );
      glBindTexture( GL_TEXTURE_2D, texture );

	  glColor4f (1.0f, 1.0f, 1.0f, 1.0f);
      glPushMatrix();
      glRotatef( 0.0f, 0.0f, 0.0f, 1.0f );
      glBegin( GL_QUADS );
      glTexCoord2d(0.0,0.0); glVertex3d(-gridX,-gridY,-0.1f);
      glTexCoord2d(1.0,0.0); glVertex3d(+gridX,-gridY,-0.1f);
      glTexCoord2d(1.0,1.0); glVertex3d(+gridX,+gridY,-0.1f);
      glTexCoord2d(0.0,1.0); glVertex3d(-gridX,+gridY,-0.1f);
      glEnd();
      glPopMatrix();
	   glDisable( GL_TEXTURE_2D );
	 glEnable (GL_LIGHTING);										//end texture, debug zz
	}

	npCameraUpdate( dataRef );

	glLoadIdentity();

	gluLookAt (	camPlot->translate.x, 
				camPlot->translate.y,
				camPlot->translate.z, 
				camPlot->translate.x + camPlot->rotate.x,
				camPlot->translate.y + camPlot->rotate.y,
				camPlot->translate.z + camPlot->rotate.z,
				upX, upY, upZ );

//	glOrtho ( -2.0f, 2.0f, -2.0f, 2.0f, 0.05f, 2.0f);
//	gluOrtho2D ( -2.0f, 2.0f, -2.0f, 2.0f);

	npDrawHUD (dataRef);				// draws overlays, grids, text...

	npCopyGPUtoCPU (dataRef);			// copies display to main CPU memory

//	glfwSleep (0.1f);
}


// copy the rasterized display to the main system memory (CPU RAM or Drive)
//------------------------------------------------------------------------------
void npCopyGPUtoCPU (void* dataRef)
{

}

//																			debug, zz
/*
//------------------------------------------------------------------------------
void npMouseEvent (int button, int state, int x, int y)
{
	pData data = (pData) npDataGetDataRef();

	previousX = x;
	previousY = y;

	if (button == GLUT_LEFT_BUTTON && state == GLUT_UP)
	{
		data->io.mouseDeltaX = 0.0f;
		data->io.mouseDeltaY = 0.0f;
	}
}

//------------------------------------------------------------------------------
void npMouseMotion (int x, int y)
{
	pData data = (pData) npDataGetDataRef();


	data->io.mouseDeltaX = (GLfloat)(x - previousX);
	data->io.mouseDeltaY = (GLfloat)(y - previousY);

//	printf ("\n %d %d", data->io.mouseDeltaX, data->io.mouseDeltaY);

	previousX = x;
	previousY = y;
}
*/	//zz-end//

//MB//

//------------------------------------------------------------------------------
void npMouseEvent (int button, int state, int x, int y)
{
	pData data = (pData) npDataGetDataRef();

	previousX = x;
	previousY = y;	
	//	WATCH_FUNC;
	WATCH_INT( button );
//	WATCH_INT( state );

	if (button == GLUT_LEFT_BUTTON && state == GLUT_UP)
	{
		data->io.mouseDeltaX = 0.0f;
		data->io.mouseDeltaY = 0.0f;
	}
	
//MB zz
	if( state == GLUT_DOWN )	//Mouse Click Down (Press)
	{
	//	nposShowCursor(false);								//zz debug buggy, why
	//	nposShowCursor(false);								//zz debug buggy, why
		buttonDown = button;
//		picking = true;
		PickPass( true );
		npGLDrawScene( data );
		y = height - y;
		npPick( x, y );
		npGLDrawScene( data );							//zz needed to select object... debug
		PickPass( false );
	}
	if( state == GLUT_UP )						//Mouse Click Up (Release)
	{	
	//	nposShowCursor(true);
	//	nposShowCursor(true);		
	//	nposSetCursorPos (500,500);							//zz debug
		picking = false;
		PickPass( false );
		buttonDown = NOBUTTON;
		cmDY = 0.0;
	}
//MB zz end
/*
	if( state == 0 )	//Mouse Click Down (Press)
	{
		//for clearing selection in single pick mode, otherwise multipick
//		if(data->io.key.modShift == false)
//		npCtrlCommand (kGraphSelectAllToggle, data);

		PickPass( true );
		npGLDrawScene( data );						//zz debug, make this run every frame
		y = height - y;
		npPick( x, y );
		npGLDrawScene( data );							//zz needed to select object... debug
		PickPass( false );								// drawScene could be replaced with ctrlCmdSelectByID, debug

	}
	else				//Mouse Click Up (Release)
	{	
		PickPass( false );
	}
*/
}

//------------------------------------------------------------------------------
void npMouseMotion (int x, int y)
{
	//MB zz
	int dX = 0;
	int dY = 0;
	static float tX = 0.0;
	static float tY = 0.0;

//	WATCH_FUNC;
	
	pData data = (pData) npDataGetDataRef();

	data->io.mouseDeltaX = (GLfloat)(x - previousX);
	data->io.mouseDeltaY = (GLfloat)(y - previousY);
	
	//	printf ("\n %d %d", data->io.mouseDeltaX, data->io.mouseDeltaY);

	if( !picking )
	{
		if (data->map.plotCount > 3)
		{
			npCtrlCommand (kGraphSelectAllToggle, data);	//zz debug not efficient
			npCtrlCommand (kGraphCamera, data);
		}

		dX = x - previousX;
		dY = y - previousY;
		if( buttonDown == GLUT_LEFT_BUTTON )
		{
			tX += (float)dX / 300.0;
			tY += (float)dY / 600.0;
			theta = tX;
			zeta  = -tY;
		}
		if( buttonDown == GLUT_RIGHT_BUTTON )
		{
			cmDX += (float)dX / 800.0;
			cmDY += (float)dY / 800.0;
			
			dX = x - previousX;
			tX += (float)dX / 300.0;
			theta = tX;
		}
	}
	

	//MB zz end
	previousX = x;
	previousY = y;
}



void npPick( int x, int y )
{
	GLubyte r,g,b,a;	//Unsigned Byte 8-bit Range: 0-255 // changed from u_int8_t, zz debug
	char pixels[4];		//4 Bytes
	int idInRGB;

	pData data;

//	WATCH_FUNC;
//	WATCH_INT( x );
//	WATCH_INT( y );
//	WATCH_INT( width );
//	WATCH_INT( height );

	data = (pData) npDataGetDataRef();

	glReadPixels( x,y, 1,1, GL_RGBA, GL_UNSIGNED_BYTE, pixels );			//Read the Pixel
	

	r = pixels[0];
	g = pixels[1];
	b = pixels[2];
	a = pixels[3];
	
//	WATCH_INT( r );
//	WATCH_INT( g );
//	WATCH_INT( b );
//	WATCH_INT( a );

	idInRGB = RGBtoID( r,g,b );
	selectedID = idInRGB;
//	int idInAlpha = a;
//	selectedID = idInAlpha;

	//MB zz
	WATCH_INT( selectedID );
	if( selectedID == 0 )
		picking = false;
	else
		picking = true;
	//MB zz end

	//Select Camera to Deselect Pin

//	data->map.plotIndex = 1;						//select camera
//	data->map.activePlot = data->map.plot[data->map.plotIndex];
}

//MB zz
void npCameraUpdate( void* dataRef )
{
	int	camIndex = 1;
	pData data = (pData) dataRef;
	NPplotPtr camPlot = data->map.plot[camIndex];
	
	camPlot->rotate.x += 1.0 * sin(theta);
	camPlot->rotate.y += 1.0 * cos(theta);
	camPlot->rotate.z += zeta;
	
	if( buttonDown == GLUT_RIGHT_BUTTON )
	{
		camPlot->translate.x += -cmDY*camPlot->rotate.x;
		camPlot->translate.y += -cmDY*camPlot->rotate.y; 
	}

//	WATCH_FLOAT( camPlot->rotate.x );
//	WATCH_FLOAT( camPlot->rotate.y );
//	WATCH_FLOAT( camPlot->rotate.z );
}

//MB-end//

